// (C) 2007-2023 GoodData Corporation
import React from "react";
import { v4 } from "uuid";
import set from "lodash/set.js";
import isEqual from "lodash/isEqual.js";
import noop from "lodash/noop.js";
import partial from "lodash/partial.js";
import throttle from "lodash/throttle.js";
import isNil from "lodash/isNil.js";
import cx from "classnames";
import { Chart } from "./Chart.js";
import { isFunnel, isPieOrDonutChart, isOneOfTypes, isHeatmap, isSankeyOrDependencyWheel, isWaterfall, } from "../chartTypes/_util/common.js";
import { VisualizationTypes } from "@gooddata/sdk-ui";
import { alignChart } from "../chartTypes/_chartCreators/helpers.js";
import { Legend, getLegendDetails, } from "@gooddata/sdk-ui-vis-commons";
import { Bubble, BubbleHoverTrigger, Icon } from "@gooddata/sdk-ui-kit";
import { BOTTOM, LEFT, RIGHT, TOP } from "../typings/mess.js";
/**
 * @internal
 */
export const FLUID_LEGEND_THRESHOLD = 768;
export function renderChart(props) {
    return React.createElement(Chart, { ...props });
}
export function renderLegend(props) {
    return React.createElement(Legend, { ...props });
}
export class HighChartsRenderer extends React.PureComponent {
    static defaultProps = {
        afterRender: noop,
        height: null,
        legend: {
            enabled: true,
            responsive: false,
            position: RIGHT,
        },
        chartRenderer: renderChart,
        legendRenderer: renderLegend,
        onLegendReady: noop,
        documentObj: document,
    };
    highchartsRendererRef = React.createRef(); // whole component = legend + chart
    chartRef;
    containerId = `visualization-${v4()}`;
    constructor(props) {
        super(props);
        this.state = {
            legendItemsEnabled: [],
            showFluidLegend: this.shouldShowFluid(),
        };
    }
    onWindowResize = () => {
        this.setState({
            showFluidLegend: this.shouldShowFluid(),
        });
        this.realignPieOrDonutChart();
    };
    throttledOnWindowResize = throttle(this.onWindowResize, 100);
    shouldShowFluid() {
        const { documentObj, legend } = this.props;
        return (documentObj.documentElement.clientWidth < FLUID_LEGEND_THRESHOLD &&
            legend?.responsive !== "autoPositionWithPopup");
    }
    UNSAFE_componentWillMount() {
        this.resetLegendState(this.props);
    }
    componentDidMount() {
        // http://stackoverflow.com/questions/18240254/highcharts-width-exceeds-container-div-on-first-load
        setTimeout(() => {
            if (this.chartRef) {
                const chart = this.chartRef.getChart();
                if (chart.container?.style) {
                    chart.container.style.height = (this.props.height && String(this.props.height)) || "100%";
                    chart.container.style.position = this.props.height ? "relative" : "absolute";
                    chart.reflow();
                }
            }
        }, 0);
        this.props.onLegendReady({
            legendItems: this.getItems(this.props.legend.items),
        });
        window.addEventListener("resize", this.throttledOnWindowResize);
    }
    componentWillUnmount() {
        this.throttledOnWindowResize.cancel();
        window.removeEventListener("resize", this.throttledOnWindowResize);
    }
    UNSAFE_componentWillReceiveProps(nextProps) {
        const thisLegendItems = this.props.legend?.items ?? [];
        const nextLegendItems = nextProps.legend?.items ?? [];
        const hasLegendChanged = !isEqual(thisLegendItems, nextLegendItems);
        if (hasLegendChanged) {
            this.resetLegendState(nextProps);
        }
        if (!isEqual(this.props.legend.items, nextProps.legend.items)) {
            this.props.onLegendReady({
                legendItems: this.getItems(nextProps.legend.items),
            });
        }
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    onLegendItemClick = (item) => {
        this.setState({
            legendItemsEnabled: set([...this.state.legendItemsEnabled], item.legendIndex, !this.state.legendItemsEnabled[item.legendIndex]),
        });
    };
    setChartRef = (chartRef) => {
        this.chartRef = chartRef;
    };
    getFlexDirection(position) {
        if (position === TOP || position === BOTTOM) {
            return "column";
        }
        return "row";
    }
    getItems(items) {
        return items.map((i) => {
            return {
                name: i.name,
                color: i.color,
                onClick: partial(this.onLegendItemClick, i),
            };
        });
    }
    resetLegendState(props) {
        const legendItemsCount = props.legend?.items?.length ?? 0;
        this.setState({
            legendItemsEnabled: new Array(legendItemsCount).fill(true),
        });
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    onChartSelection = (event) => {
        const chartWrapper = event.target.renderTo.parentElement;
        const resetZoomButton = chartWrapper.closest(".visualization").querySelector(".viz-zoom-out");
        if (event.resetSelection) {
            resetZoomButton.style.display = "none";
        }
        else {
            resetZoomButton.style.display = "grid";
        }
        return undefined;
    };
    skipLeadingZeros(values) {
        const result = [...values];
        while (result[0]?.y === 0) {
            result.shift();
        }
        return result;
    }
    createChartConfig(chartConfig, legendItemsEnabled) {
        const { series, chart, xAxis, yAxis } = chartConfig;
        const selectionEvent = chart.zoomType
            ? {
                selection: this.onChartSelection,
            }
            : {};
        const firstSeriesTypes = [
            VisualizationTypes.PIE,
            VisualizationTypes.DONUT,
            VisualizationTypes.TREEMAP,
            VisualizationTypes.FUNNEL,
            VisualizationTypes.PYRAMID,
        ];
        const multipleSeries = isOneOfTypes(chart.type, firstSeriesTypes);
        let items = isOneOfTypes(chart.type, firstSeriesTypes) ? series?.[0]?.data : series;
        if (isFunnel(chart.type)) {
            items = this.skipLeadingZeros(items).filter((i) => !isNil(i.y));
        }
        const updatedItems = items.map((item, itemIndex) => {
            const visible = legendItemsEnabled[itemIndex] !== undefined ? legendItemsEnabled[itemIndex] : true;
            return {
                ...item,
                visible: isNil(item.visible) ? visible : item.visible,
            };
        });
        let updatedSeries = updatedItems;
        if (multipleSeries) {
            updatedSeries = [
                {
                    ...series?.[0],
                    data: updatedItems,
                },
                ...series.slice(1),
            ];
        }
        return {
            ...chartConfig,
            chart: {
                ...chartConfig?.chart,
                events: {
                    ...chartConfig?.chart?.events,
                    ...selectionEvent,
                },
            },
            series: updatedSeries,
            yAxis: yAxis.map((ax) => ({
                ...ax,
                title: {
                    ...ax?.title,
                    style: {
                        ...ax?.title?.style,
                        textOverflow: "ellipsis",
                        overflow: "hidden",
                    },
                },
            })),
            // perform a shallow copy of axis
            // (otherwise there's a highcharts internal error on smallest responsive charts)
            xAxis: xAxis.map((ax) => ({
                ...ax,
            })),
        };
    }
    renderLegend(legendDetails, contentRect, containerId) {
        const { chartOptions, legend, height, legendRenderer, locale } = this.props;
        const { items, format } = legend;
        const { showFluidLegend } = this.state;
        if (!legend.enabled) {
            return null;
        }
        let { type } = chartOptions;
        if (isPieOrDonutChart(type)) {
            type = VisualizationTypes.PIE;
        }
        const onItemClick = isSankeyOrDependencyWheel(type) || isWaterfall(type) ? noop : this.onLegendItemClick;
        const legendProps = {
            responsive: legend.responsive,
            enableBorderRadius: legend.enableBorderRadius,
            seriesMapper: legend.seriesMapper,
            series: items,
            onItemClick,
            legendItemsEnabled: this.state.legendItemsEnabled,
            heatmapLegend: isHeatmap(type),
            height,
            legendLabel: legendDetails?.name,
            maximumRows: legendDetails?.maxRows,
            position: legendDetails.position,
            format,
            locale,
            showFluidLegend,
            validateOverHeight: noop,
            contentDimensions: contentRect?.client,
            containerId,
        };
        return legendRenderer(legendProps);
    }
    renderHighcharts() {
        // shrink chart to give space to legend items
        const style = { flex: "1 1 auto", position: "relative", overflow: "hidden" };
        const config = this.createChartConfig(this.props.hcOptions, this.state.legendItemsEnabled);
        const chartProps = {
            domProps: { className: "viz-react-highchart-wrap gd-viz-highchart-wrap", style },
            ref: this.setChartRef,
            config,
            callback: this.props.afterRender,
        };
        return this.props.chartRenderer(chartProps);
    }
    onZoomOutButtonClick = () => {
        this.chartRef.getChart().zoomOut();
    };
    renderZoomOutButton() {
        const { hcOptions: { chart }, theme, resetZoomButtonTooltip, } = this.props;
        if (chart?.zoomType) {
            return (React.createElement(BubbleHoverTrigger, { tagName: "abbr", hideDelay: 100, showDelay: 100, className: "gd-bubble-trigger-zoom-out" },
                React.createElement("button", { className: "viz-zoom-out s-zoom-out", onClick: this.onZoomOutButtonClick, style: { display: "none" } },
                    React.createElement(Icon.Undo, { width: 20, height: 20, color: theme?.palette?.complementary?.c7 })),
                React.createElement(Bubble, { alignPoints: [{ align: "cr cl" }, { align: "cl cr" }] }, resetZoomButtonTooltip)));
        }
        return null;
    }
    renderVisualization() {
        const { legend, chartOptions, contentRect } = this.props;
        const legendDetailOptions = {
            showFluidLegend: this.state.showFluidLegend,
            contentRect,
            isHeatmap: isHeatmap(chartOptions.type),
            legendLabel: chartOptions.legendLabel,
        };
        const legendDetails = getLegendDetails(legend.position, legend.responsive, legendDetailOptions);
        if (!legendDetails) {
            return null;
        }
        const classes = cx("viz-line-family-chart-wrap", "s-viz-line-family-chart-wrap", legend.responsive === true ? "responsive-legend" : "non-responsive-legend", {
            [`flex-direction-${this.getFlexDirection(legendDetails.position)}`]: true,
            "legend-position-bottom": legendDetails.position === BOTTOM,
        }, this.containerId);
        const legendPosition = legendDetails.position;
        const isLegendRenderedFirst = legendPosition === TOP || legendPosition === LEFT || this.state.showFluidLegend;
        return (React.createElement("div", { className: classes, ref: this.highchartsRendererRef },
            this.renderZoomOutButton(),
            isLegendRenderedFirst
                ? this.renderLegend(legendDetails, contentRect, this.containerId)
                : null,
            this.renderHighcharts(),
            !isLegendRenderedFirst
                ? this.renderLegend(legendDetails, contentRect, this.containerId)
                : null));
    }
    render() {
        return this.renderVisualization();
    }
    realignPieOrDonutChart() {
        const { chartOptions: { type, verticalAlign }, } = this.props;
        const { chartRef } = this;
        if (isPieOrDonutChart(type) && chartRef) {
            alignChart(chartRef.getChart(), verticalAlign);
        }
    }
}
//# sourceMappingURL=HighChartsRenderer.js.map